Decrypt_Go_RC6:
	B_CALL GrBufClr
	B_CALL ClrLCDFull
	ld hl,0
	ld (pencol),hl
	ld hl,Decrypt_Go_RC6_Status
	call vputstr
	;Now we can load the real key and decode it
	ld hl,KeyFile
	rst 20h
	B_CALL ChkFindSym
	jr c,Decrypt_Go_RC6_KeyNotFound
	ld a,b
	or a
	jr z,Decrypt_Go_RC6_KeyLoad
	;Key appvar is archived...Stream to flash
	;A is ROM page
	ex de,hl
	ld de,20
	add hl,de
	bit 7,h
	jr z,Decrypt_Go_RC6_ArchivedGo
	res 7,h
	set 6,h
	inc a
Decrypt_Go_RC6_ArchivedGo:
	;HL, A are set
	ld de,8000h
	ld bc,32
	B_CALL FlashToRam
	jr Decrypt_Go_RC6_DecodeKey
Decrypt_Go_RC6_KeyLoad:
	ex de,hl
	inc hl
	inc hl
	ld de,8000h
	ld bc,32
	ldir
	;Key now copied to RAM at 8000h
Decrypt_Go_RC6_DecodeKey:
	ld hl,8000h
	call GenerateKeySchedule
	;User key now ready for decrypt. Let's go.
	ld hl,(curpointer)
	ld de,-5
	add hl,de
	ld a,(hl)
	or a
	jr z,Decrypt_Go_RC6_ReadyToDecrypt
	push hl
	B_CALL ZeroOP1
	pop hl
	ld de,5
	add hl,de
	ld a,(hl)
	ld (OP1),a
	ld de,-6
	add hl,de
	ld b,(hl)
	dec hl
	ld de,OP1+1
Decrypt_Go_RC6_OP1Load:
	ld a,(hl)
	ld (de),a
	dec hl
	inc de
	djnz Decrypt_Go_RC6_OP1Load
	B_CALL Arc_Unarc
Decrypt_Go_RC6_ReadyToDecrypt:
	ld hl,(curpointer)
	dec hl
	dec hl
	dec hl
	ld e,(hl)
	dec hl
	ld d,(hl)
	ex de,hl
	;HL is size bytes
	ld c,(hl)
	inc hl
	ld b,(hl)
	ld de,14
	add hl,de
	push hl
	ld h,b
	ld l,c
	ld bc,13
	or a
	sbc hl,bc
	ld b,h
	ld c,l
	pop hl
	;HL is start, BC is size.
	srl b
	rr c
	srl b
	rr c
	;HL is start, BC number of times
	ld (total),bc
	ld ix,0
	ld (status),ix
	push bc
	push hl
	ld de,5*256+2
	ld (currow),de
	ld a,' '
	B_CALL PutC
	ld a,'/'
	B_CALL PutC
	ld hl,(total)
	B_CALL DispHL
	pop hl
	pop bc
Decrypt_Go_RC6_Loop:
	push bc
	call RC6_Decrypt
	push de
	;Update status
	ld hl,(status)
	inc hl
	ld (status),hl
	bit 3,l
	jp z,Decrypt_Go_RC6_Loop_NoShowStatus
	ld de,2
	ld (currow),de
	B_CALL DispHL
Decrypt_Go_RC6_Loop_NoShowStatus:
	pop hl
	pop bc
	dec bc
	ld a,b
	or c
	jp nz,Decrypt_Go_RC6_Loop
	res protected,(iy+progSearch)		;Was it protected before?
	ld hl,(curpointer)
	dec hl
	dec hl
	dec hl
	ld e,(hl)
	dec hl
	ld d,(hl)
	ex de,hl
	;HL is start of variable, size bytes
	ld e,(hl)
	inc hl
	ld d,(hl)
	push de		;DE is current size
	ld de,11
	add hl,de
	bit 7,(hl)
	jr z,Decrypt_Go_RC6_SkipSet
	set protected,(iy+progSearch)
Decrypt_Go_RC6_SkipSet:
	;HL over old size bytes -1
	inc hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	;BC is old size byte
	pop hl
	;HL new, BC old
	or a
	sbc hl,bc
	push bc
	;HL is how many were added
	ex de,hl
	;DE is how many to delete
	ld hl,(curpointer)
	dec hl
	dec hl
	dec hl
	ld c,(hl)
	dec hl
	ld b,(hl)
	ld h,b
	ld l,c
	;HL now is program start
	inc hl
	inc hl
	push hl
	B_CALL DelMem
	pop hl
	dec hl
	dec hl
	pop bc
	ld (hl),c
	inc hl
	ld (hl),b
	;Size bytes now fixed
	;The last thing is to fix the 5 or 6
	bit protected,(iy+progSearch)
	jp nz,AppStart
	ld hl,(curpointer)
	ld (hl),5
	jp AppStart
Decrypt_Go_RC6_KeyNotFound:
	ld hl,Decrypt_Go_RC6_KeyNotFound_Data
	call DisplayScreen
	jp AppStart
Decrypt_Go_RC6_KeyNotFound_Data:
	DB "The key appvar could",1
	DB "not be found.",1,1,1,1,1
	DB "Press any key",0
Decrypt_Go_RC6_Status:
	DB "Decrypting (256-bit RC6)...",0
Decrypt_Go_Blowfish_Status:
	DB "Decrypting (Blowfish)...",0
Decrypt_Go_Blowfish:
	B_CALL GrBufClr
	B_CALL ClrLCDFull
	ld hl,0
	ld (pencol),hl
	ld hl,Encrypt_Go_Blowfish_SetupStatus
	call vputstr
	call Blowfish_Setup
	B_CALL ClrLCDFull
	ld hl,0
	ld (pencol),hl
	ld hl,Decrypt_Go_Blowfish_Status
	call vputstr
	ld hl,(curpointer)
	ld de,-5
	add hl,de
	ld a,(hl)
	or a
	jr z,Decrypt_Go_Blowfish_ReadyToDecrypt
	push hl
	B_CALL ZeroOP1
	pop hl
	ld de,5
	add hl,de
	ld a,(hl)
	ld (OP1),a
	ld de,-6
	add hl,de
	ld b,(hl)
	dec hl
	ld de,OP1+1
Decrypt_Go_Blowfish_OP1Load:
	ld a,(hl)
	ld (de),a
	dec hl
	inc de
	djnz Decrypt_Go_Blowfish_OP1Load
	B_CALL Arc_Unarc
Decrypt_Go_Blowfish_ReadyToDecrypt:
	ld hl,(curpointer)
	dec hl
	dec hl
	dec hl
	ld e,(hl)
	dec hl
	ld d,(hl)
	ex de,hl
	;HL is size bytes
	ld c,(hl)
	inc hl
	ld b,(hl)
	ld de,14
	add hl,de
	push hl
	ld h,b
	ld l,c
	ld bc,13
	or a
	sbc hl,bc
	ld b,h
	ld c,l
	pop hl
	;HL is start, BC is size.
	srl b
	rr c
	srl b
	rr c
	srl b
	rr c
	;HL is start, BC number of times
	ld d,h
	ld e,l
	ld (total),bc
	ld ix,0
	ld (status),ix
	push bc
	push de
	push hl
	ld de,5*256+2
	ld (currow),de
	ld a,' '
	B_CALL PutC
	ld a,'/'
	B_CALL PutC
	ld hl,(total)
	B_CALL DispHL
	pop hl
	pop de
	pop bc
Decrypt_Go_Blowfish_Loop:
	push bc
	push de
	push hl
	ld de,cbccode
	ld bc,8
	ldir
	pop hl
	pop de
	push de
	push hl
	ld a,1
	call Blowfish_Process
	pop hl
	ld de,cbcbytes
	ld b,8
Decrypt_Go_Blowfish_Loop_CBC:
	ld a,(de)
	xor (hl)
	ld (hl),a
	inc hl
	inc de
	djnz Decrypt_Go_Blowfish_Loop_CBC
	ld hl,cbccode
	ld de,cbcbytes
	ld bc,8
	ldir
	;Update status
	ld hl,(status)
	inc hl
	ld (status),hl
	bit 3,l
	jp z,Decrypt_Go_Blowfish_Loop_NoShowStatus
	ld de,2
	ld (currow),de
	B_CALL DispHL
Decrypt_Go_Blowfish_Loop_NoShowStatus:
	pop de
	ex de,hl
	ld bc,8
	add hl,bc
	ld d,h
	ld e,l
	pop bc
	dec bc
	ld a,b
	or c
	jp nz,Decrypt_Go_Blowfish_Loop
	res protected,(iy+progSearch)		;Was it protected before?
	ld hl,(curpointer)
	dec hl
	dec hl
	dec hl
	ld e,(hl)
	dec hl
	ld d,(hl)
	ex de,hl
	;HL is start of variable, size bytes
	ld e,(hl)
	inc hl
	ld d,(hl)
	push de		;DE is current size
	ld de,11
	add hl,de
	bit 7,(hl)
	jr z,Decrypt_Go_Blowfish_SkipSet
	set protected,(iy+progSearch)
Decrypt_Go_Blowfish_SkipSet:
	;HL over old size bytes -1
	inc hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	;BC is old size byte
	pop hl
	;HL new, BC old
	or a
	sbc hl,bc
	push bc
	;HL is how many were added
	ex de,hl
	;DE is how many to delete
	ld hl,(curpointer)
	dec hl
	dec hl
	dec hl
	ld c,(hl)
	dec hl
	ld b,(hl)
	ld h,b
	ld l,c
	;HL now is program start
	inc hl
	inc hl
	push hl
	B_CALL DelMem
	pop hl
	dec hl
	dec hl
	pop bc
	ld (hl),c
	inc hl
	ld (hl),b
	;Size bytes now fixed
	;The last thing is to fix the 5 or 6
	call Blowfish_Close
	bit protected,(iy+progSearch)
	jp nz,AppStart
	ld hl,(curpointer)
	ld (hl),5
	jp AppStart
Decrypt_Go_GOST_Status:
	DB "Decrypting (GOST)...",0
Decrypt_Go_GOST:
	B_CALL GrBufClr
	B_CALL ClrLCDFull
	ld hl,0
	ld (pencol),hl
	ld hl,Decrypt_Go_GOST_Status
	call vputstr
	call GOST_Setup
	ld hl,(curpointer)
	ld de,-5
	add hl,de
	ld a,(hl)
	or a
	jr z,Decrypt_Go_GOST_ReadyToDecrypt
	push hl
	B_CALL ZeroOP1
	pop hl
	ld de,5
	add hl,de
	ld a,(hl)
	ld (OP1),a
	ld de,-6
	add hl,de
	ld b,(hl)
	dec hl
	ld de,OP1+1
Decrypt_Go_GOST_OP1Load:
	ld a,(hl)
	ld (de),a
	dec hl
	inc de
	djnz Decrypt_Go_GOST_OP1Load
	B_CALL Arc_Unarc
Decrypt_Go_GOST_ReadyToDecrypt:
	ld hl,(curpointer)
	dec hl
	dec hl
	dec hl
	ld e,(hl)
	dec hl
	ld d,(hl)
	ex de,hl
	;HL is size bytes
	ld c,(hl)
	inc hl
	ld b,(hl)
	ld de,14
	add hl,de
	push hl
	ld h,b
	ld l,c
	ld bc,13
	or a
	sbc hl,bc
	ld b,h
	ld c,l
	pop hl
	;HL is start, BC is size.
	srl b
	rr c
	srl b
	rr c
	srl b
	rr c
	;HL is start, BC number of times
	ld (total),bc
	ld ix,0
	ld (status),ix
	push bc
	push hl
	ld de,5*256+2
	ld (currow),de
	ld a,' '
	B_CALL PutC
	ld a,'/'
	B_CALL PutC
	ld hl,(total)
	B_CALL DispHL
	pop hl
	pop bc
	di
Decrypt_Go_GOST_Loop:
	push bc
	push hl
	ld de,cbccode
	ld bc,8
	ldir
	pop hl
	push hl
	call GOST_Decrypt
	pop hl
	push de
	ld de,cbcbytes
	ld b,8
Decrypt_Go_GOST_Loop_CBC:
	ld a,(de)
	xor (hl)
	ld (hl),a
	inc hl
	inc de
	djnz Decrypt_Go_GOST_Loop_CBC
	ld hl,cbccode
	ld de,cbcbytes
	ld bc,8
	ldir
	;Update status
	ld hl,(status)
	inc hl
	ld (status),hl
	bit 3,l
	jp z,Decrypt_Go_GOST_Loop_NoShowStatus
	ld de,2
	ld (currow),de
	B_CALL DispHL
Decrypt_Go_GOST_Loop_NoShowStatus:
	pop hl
	pop bc
	dec bc
	ld a,b
	or c
	jp nz,Decrypt_Go_GOST_Loop
	res protected,(iy+progSearch)		;Was it protected before?
	ld hl,(curpointer)
	dec hl
	dec hl
	dec hl
	ld e,(hl)
	dec hl
	ld d,(hl)
	ex de,hl
	;HL is start of variable, size bytes
	ld e,(hl)
	inc hl
	ld d,(hl)
	push de		;DE is current size
	ld de,11
	add hl,de
	bit 7,(hl)
	jr z,Decrypt_Go_GOST_SkipSet
	set protected,(iy+progSearch)
Decrypt_Go_GOST_SkipSet:
	;HL over old size bytes -1
	inc hl
	ld c,(hl)
	inc hl
	ld b,(hl)
	;BC is old size byte
	pop hl
	;HL new, BC old
	or a
	sbc hl,bc
	push bc
	;HL is how many were added
	ex de,hl
	;DE is how many to delete
	ld hl,(curpointer)
	dec hl
	dec hl
	dec hl
	ld c,(hl)
	dec hl
	ld b,(hl)
	ld h,b
	ld l,c
	;HL now is program start
	inc hl
	inc hl
	push hl
	B_CALL DelMem
	pop hl
	dec hl
	dec hl
	pop bc
	ld (hl),c
	inc hl
	ld (hl),b
	;Size bytes now fixed
	;The last thing is to fix the 5 or 6
	bit protected,(iy+progSearch)
	jp nz,AppStart
	ld hl,(curpointer)
	ld (hl),5
	ei
	jp AppStart
