GOST_Setup:
	ld hl,KeyFile
	rst 20h
	B_CALL ChkFindSym
	jr c,Encrypt_Go_RC6_KeyNotFound
	ld a,b
	or a
	jr z,GOST_Setup_KeyLoad
	;Key appvar is archived...Stream to flash
	;A is ROM page
	ex de,hl
	ld de,20
	add hl,de
	bit 7,h
	jr z,GOST_Setup_KeyArchivedGo
	res 7,h
	set 6,h
	inc a
GOST_Setup_KeyArchivedGo:
	;HL, A are set
	ld de,8000h
	ld bc,32
	B_CALL FlashToRam
	jr GOST_Setup_KeyDone
GOST_Setup_KeyLoad:
	ex de,hl
	inc hl
	inc hl
	ld de,8000h
	ld bc,32
	ldir
	;Key now copied to RAM at 8000h
GOST_Setup_KeyDone:
	;Now 8000h contains the key
	ld hl,8000h
	ld de,kArray
	ld bc,32
	ldir
	ret
GOST_Decrypt:
	;Input - HL points to block to decrypt
	;Output - HL points to decrypted block
	push hl
	ld de,R1
	ld bc,8
	ldir
	ld a,1
	ld (vJ),a
GOST_Decrypt_Loop:
	ld hl,R1
	ld de,RTemp
	ld bc,4
	ldir
	ld a,(vJ)
	dec a
	and 7
	add a,a
	add a,a
	ld h,99h
	ld l,a
	ld de,RTemp
	call GOST_Add_HL_DE
	call GOST_SGenerate
	call GOST_Rotate11
	call GOST_XOR_RTemp_R2
	ld hl,R1
	ld de,R2
	ld bc,4
	ldir
	ld hl,RTemp
	ld de,R1
	ld bc,4
	ldir
	ld a,(vJ)
	inc a
	ld (vJ),a
	cp 9
	jr nz,GOST_Decrypt_Loop
	;Now for second loop
	ld a,9
	ld (vJ),a
GOST_Decrypt_Loop2:
	ld hl,R1
	ld de,RTemp
	ld bc,4
	ldir
	ld a,(vJ)
	ld b,a
	ld a,32
	sub b
	and 7
	add a,a
	add a,a
	ld h,99h
	ld l,a
	ld de,RTemp
	call GOST_Add_HL_DE
	call GOST_SGenerate
	call GOST_Rotate11
	call GOST_XOR_RTemp_R2
	ld hl,R1
	ld de,R2
	ld bc,4
	ldir
	ld hl,RTemp
	ld de,R1
	ld bc,4
	ldir
	ld a,(vJ)
	inc a
	ld (vJ),a
	cp 32
	jr nz,GOST_Decrypt_Loop2
	ld hl,9900h
	ld de,RTemp
	call GOST_Add_HL_DE
	call GOST_SGenerate
	call GOST_Rotate11
	call GOST_XOR_RTemp_R2
	ld hl,RTemp
	ld de,R2
	ld bc,4
	ldir
	pop de		;HL as DE
	ld hl,R1
	ld bc,8
	ldir
	ret
GOST_Encrypt:
	;Input - HL points to block to encrypt
	;Output - HL points to decrypted block
	push hl		;Save for later
	ld de,R1
	ld bc,8
	ldir
	;R1 and R2 initialized
	;Setup loop
	ld a,1
	ld (vJ),a
GOST_Encrypt_Loop:
	ld hl,R1
	ld de,RTemp
	ld bc,4
	ldir
	ld a,(vJ)
	dec a
	and 7
	add a,a
	add a,a
	ld h,99h
	ld l,a
	;HL points to K(j-1)
	ld de,RTemp
	;Now add them
	call GOST_Add_HL_DE
	;a(j-1) + K(j-1) mod 8 completed...
	call GOST_SGenerate
	;RTemp now has the S-box results
	;Rotate 11 bits left
	call GOST_Rotate11
	call GOST_XOR_RTemp_R2
	ld hl,R1
	ld de,R2
	ld bc,4
	ldir
	ld hl,RTemp
	ld de,R1
	ld bc,4
	ldir
	ld a,(vJ)
	inc a
	ld (vJ),a
	cp 25
	jr nz,GOST_Encrypt_Loop
	;Now for part 2...
	ld a,25
	ld (vJ),a
GOST_Encrypt_Loop2:
	ld hl,R1
	ld de,RTemp
	ld bc,4
	ldir
	ld a,(vJ)
	ld b,a
	ld a,32
	sub b
	add a,a
	add a,a
	ld h,99h
	ld l,a
	;HL points to K(32-j)
	ld de,RTemp
	call GOST_Add_HL_DE
	call GOST_SGenerate
	call GOST_Rotate11
	call GOST_XOR_RTemp_R2
	ld hl,R1
	ld de,R2
	ld bc,4
	ldir
	ld hl,RTemp
	ld de,R1
	ld bc,4
	ldir
	ld a,(vJ)
	inc a
	ld (vJ),a
	cp 32
	jr nz,GOST_Encrypt_Loop2
	;Now do for 32
	ld hl,R1
	ld de,RTemp
	ld bc,4
	ldir
	ld hl,9900h
	ld de,RTemp
	call GOST_Add_HL_DE
	call GOST_SGenerate
	call GOST_Rotate11
	call GOST_XOR_RTemp_R2
	ld hl,RTemp
	ld de,R2
	ld bc,4
	ldir
	pop de
	ld hl,R1
	ld bc,8
	ldir
	ret
GOST_Add_HL_DE:
	ld a,(de)
	add a,(hl)
	ld (de),a
	inc hl
	inc de
	ld a,(de)
	adc a,(hl)
	ld (de),a
	inc hl
	inc de
	ld a,(de)
	adc a,(hl)
	ld (de),a
	inc hl
	inc de
	ld a,(de)
	adc a,(hl)
	ld (de),a
	ret
GOST_XOR_RTemp_R2:
	ld hl,R2
	ld de,RTemp
	;Now do XOR
	ld a,(de)
	xor (hl)
	ld (de),a
	inc hl
	inc de
	ld a,(de)
	xor (hl)
	ld (de),a
	inc hl
	inc de
	ld a,(de)
	xor (hl)
	ld (de),a
	inc hl
	inc de
	ld a,(de)
	xor (hl)
	ld (de),a
	ret
GOST_Rotate11:
	ld a,(RTemp)
	ld hl,RTemp+1
	ld de,RTemp
	ld bc,3
	ldir
	ld (RTemp+3),a
	;Now 3 more rotates
	ld hl,(RTemp)
	ld de,(RTemp+2)
;input: low word in HL, high word in DE
;output: shifted left by 3
	xor a
	add hl,hl
	adc a,a
	add hl,hl
	adc a,a
	add hl,hl
	adc a,a
	ex de,hl
	ld c,a
	xor a
	add hl,hl
	adc a,a
	add hl,hl
	adc a,a
	add hl,hl
	adc a,a
	ex de,hl
	or l
	ld l,a
	ld a,c
	or e
	ld e,a
	ld (RTemp),hl
	ld (RTemp+2),de
	ret
GOST_SGenerate:
	;Replace RTemp with its s-box output
	ld a,(RTemp)
	ld d,a
	and 00001111b
	ld hl,GOST_S_Box1
	ld b,0
	ld c,a
	add hl,bc
	ld a,(hl)
	and 00001111b
	ld e,a
	ld a,d
	and 11110000b
	sra a
	sra a
	sra a
	sra a
	ld hl,GOST_S_Box1
	ld c,a
	add hl,bc
	ld a,(hl)
	and 00001111b
	sla a
	sla a
	sla a
	sla a
	or e
	ld (RTemp),a
	;next byte
	ld a,(RTemp+1)
	ld d,a
	and 00001111b
	ld hl,GOST_S_Box1
	ld b,0
	ld c,a
	add hl,bc
	ld a,(hl)
	and 00001111b
	ld e,a
	ld a,d
	and 11110000b
	sra a
	sra a
	sra a
	sra a
	ld hl,GOST_S_Box1
	ld c,a
	add hl,bc
	ld a,(hl)
	and 00001111b
	sla a
	sla a
	sla a
	sla a
	or e
	ld (RTemp+1),a
	;next byte
	ld a,(RTemp+2)
	ld d,a
	and 00001111b
	ld hl,GOST_S_Box1
	ld b,0
	ld c,a
	add hl,bc
	ld a,(hl)
	and 00001111b
	ld e,a
	ld a,d
	and 11110000b
	sra a
	sra a
	sra a
	sra a
	ld hl,GOST_S_Box1
	ld c,a
	add hl,bc
	ld a,(hl)
	and 00001111b
	sla a
	sla a
	sla a
	sla a
	or e
	ld (RTemp+2),a
	;last byte
	ld a,(RTemp+3)
	ld d,a
	and 00001111b
	ld hl,GOST_S_Box1
	ld b,0
	ld c,a
	add hl,bc
	ld a,(hl)
	and 00001111b
	ld e,a
	ld a,d
	and 11110000b
	sra a
	sra a
	sra a
	sra a
	ld hl,GOST_S_Box1
	ld c,a
	add hl,bc
	ld a,(hl)
	and 00001111b
	sla a
	sla a
	sla a
	sla a
	or e
	ld (RTemp+3),a
	ret
