; RC6 Encryption
; 
; RC6 is the property, copyright, and trademark of RSA Laboratories. This particular Z80
; implementation is by Michael Vincent.
;
; The following subroutines contain the RC6 algorithm, everything else is the property of
; Michael Vincent:
; 
; GenerateKeySchedule
; RC6_Encrypt
; RC6_Decrypt
;
vA EQU B
vB EQU C
vC EQU D
vD EQU E
RC6_Decrypt:
	di
	;Input - HL points to text to decrypt (4 byte chunk)
	;Input - HL points to output location for plaintext
	;Output - Text decrypted with (rkeys) and outputted
	push hl		;Not used until end
	;We must put the 4 bytes at HL into A,B,C,D
	ld a,(hl)
	exx
	ld vA,a
	exx
	inc hl
	ld a,(hl)
	exx
	ld vB,a
	exx
	inc hl
	ld a,(hl)
	exx
	ld vC,a
	exx
	inc hl
	ld a,(hl)
	exx
	ld vD,a
	;A,B,C,D initialized
	;Now for C=C-S[2r+3]
	;Remember r=20 so S[2r+3] = S[43]
	ld a,(rkeys+43)
	ld l,a
	ld a,vC
	sub l
	ld vC,a
	;Done.
	;Now do A=A-S[2r+2]
	ld a,(rkeys+42)
	ld l,a
	ld a,vA
	sub l
	ld vA,a
	exx
	;Done.
	;Now start loop
	ld a,20		;r
	ld (vI),a
RC6_Decrypt_Loop:
	;Now do (A,B,C,D)=(D,A,B,C)
	exx
	ld l,vA
	ld vA,vD
	ld h,vB
	ld vB,l
	ld l,vC
	ld vC,h
	ld vD,l
	;Parallel assignment complete.
	;Now for U = (D * (2D+1)) <<< lg w
	ld a,vD
	exx
	add a,a
	inc a		;2D+1
	ld b,a
	exx
	ld a,vD
	exx
	call Multiply
	;A is D * (2D+1)
	rlca
	rlca
	rlca
	ld (vU),a
	;Done.
	;Now do T = (B * (2B+1)) <<< lg w
	exx
	ld a,vB
	exx
	add a,a
	inc a		;2B+1
	ld b,a
	exx
	ld a,vB
	exx
	call Multiply
	;A is B*(2B+1)
	rlca
	rlca
	rlca
	ld (vT),a
	;Done.
	;Now do C=((C-S[2i+1]) >>> t) xor u
	exx
	ld a,(vI)
	add a,a
	inc a		;A = 2i+1
	ld h,99h
	ld l,a
	ld a,vC
	sub (hl)		;C = C - S[2i+1]
	exx
	;Now we need to rotate T to the right by (and %00000111 T bits)
	ex af,af'
	ld a,(vT)
	and 00000111b
	ld b,a
	ex af,af'
RC6_Decrypt_C_Loop:
	rrca
	djnz RC6_Decrypt_C_Loop
	ld b,a
	ld a,(vU)
	xor b
	exx
	ld vC,a
	;Done with that.
	;Now do A = ((A - S[2i] >>> u) xor t
	ld a,(vI)
	add a,a
	ld h,99h
	ld l,a
	ld a,vA
	sub (hl)	;A-S[2i]
	exx
	ex af,af'
	ld a,(vU)
	and 00000111b
	ld b,a
	ex af,af'
RC6_Decrypt_A_Loop:
	rrca
	djnz RC6_Decrypt_A_Loop
	;A now is ready.
	;XOR with T
	ld b,a
	ld a,(vT)
	xor b
	exx
	ld vA,a
	exx
	;
	ld a,(vI)
	dec a
	ld (vI),a
	or a
	jp nz,RC6_Decrypt_Loop
	;Loop done
	;Now D = D - S[1]
	exx
	ld a,(rkeys+1)
	ld l,a
	ld a,vD
	sub l
	ld vD,a
	;Now B = B - S[0]
	ld a,(rkeys)
	ld l,a
	ld a,vB
	sub l
	ld vB,a
	;We are done...Now output A,B,C,D
	exx
	pop de
	exx
	ld a,vA
	exx
	ld (de),a
	inc de
	exx
	ld a,vB
	exx
	ld (de),a
	inc de
	exx
	ld a,vC
	exx
	ld (de),a
	inc de
	exx
	ld a,vD
	exx
	ld (de),a
	inc de
	ei
	ret
RC6_Encrypt:
	;Input - HL points to text to encrypt (4 byte chunk)
	;Input - HL points to output location for encrypted text
	;Output - Text encrypted with (rkeys) and outputted
	push hl		;We won't use this until at the end
	;We must put the 4 bytes at HL into A,B,C,D
	;
	;We will now be simulating vA,vB,vC,vD
	;vA = B'
	;vB = C'
	;vC = D'
	;vD = E'
	di
	ld a,(hl)
	exx
	ld vA,a	;ld (vA),a
	exx
	inc hl
	ld a,(hl)
	exx
	ld vB,a	;ld (vB),a
	exx
	inc hl
	ld a,(hl)
	exx
	ld vC,a	;ld (vC),a
	exx
	inc hl
	ld a,(hl)
	exx
	ld vD,a	;ld (vD),a
	exx
	;A,B,C,D now initialzed in accordance with inputs 
	ld a,(rkeys)
	exx
	add a,vB
	ld vB,a	;B=B+S[O]
	ld a,(rkeys+1)
	add a,vD
	ld vD,a	;D=D+S[1]
	exx
	ld a,1
	ld (vI),a
RC6_Encrypt_Loop:
	;Now do t = (B * (2B+1)) <<< lg w [lg w is 3]
	exx
	ld a,vB
	exx
	add a,a
	inc a	;2B+1
	;Now need to multiply register A * (vB)
	;
	ld b,a		;Register B = (2B+1)
	exx
	ld a,vB
	exx
	call Multiply	;Register A = (2B+1)*B
	;Now rotate result by <<< lg w (3)
	rlca
	rlca
	rlca
	ld (vT),a
	;Done with t = (B * (2B+1)) <<< lg w
	;Now below is u = (D * (2D+1) <<< lg w
	exx
	ld a,vD
	exx
	add a,a
	inc a	;2D+1
	ld b,a
	exx
	ld a,vD
	exx
	call Multiply
	rlca
	rlca
	rlca
	ld (vU),a
	;Done with u = (D * (2D+1) <<< lg w
	;Now for A = ((A xor T) <<< u) + S[2i]
	ld a,(vT)
	exx
	xor vA
	exx
	;A is A xor T
	ld c,a
	ld a,(vU)
	and 00000111b
	ld b,a
	ld a,c
RC6_Encrypt_A_Loop:
	rlca
	djnz RC6_Encrypt_A_Loop
	;Now add S[2i]
	exx
	ld vA,a
	ld a,(vI)
	add a,a
	ld h,99h
	ld l,a
	ld a,vA
	add a,(hl)		;Add the S[2i]
	ld vA,a
	;Done with A = ((A xor T) <<< u) + S[2i]
	;Now for C = ((C xor U) <<< t) + S[2i+1]
	ld a,(vU)
	xor vC		;C xor U = A
	exx
	ld c,a
	ld a,(vT)
	and 00000111b
	ld b,a
	ld a,c
RC6_Encrypt_C_Loop:
	rlca
	djnz RC6_Encrypt_C_Loop
	exx
	ld vC,a	;Get ready to add S[2i+1]
	ld a,(vI)
	add a,a
	inc a
	ld l,a
	ld h,99h
	ld a,vC
	add a,(hl)
	ld vC,a	;Done...
	;Now do (A,B,C,D) = (B,C,D,A)
	ld l,vA
	ld vA,vB
	ld vB,vC
	ld vC,vD
	ld vD,l
	;Done with parallel assignment
	;
	exx
	ld a,(vI)
	inc a
	ld (vI),a
	cp 21
	jp nz,RC6_Encrypt_Loop
	;Loop done
	;Now for A = A + S[2r+2]
	;r = 20, thus S[2r+2] = S[42]
	exx
	ld hl,rkeys+42
	ld a,vA
	add a,(hl)
	ld vA,a
	;Done with A=A+S[2r+2]
	;Now do C=C+S[2r+3]
	inc hl
	ld a,vC
	add a,(hl)
	ld vC,a
	;All done, now copy to DE for output (4 bytes)
	exx
	pop de
	exx
	ld a,vA
	exx
	ld (de),a
	inc de
	exx
	ld a,vB
	exx
	ld (de),a
	inc de
	exx
	ld a,vC
	exx
	ld (de),a
	inc de
	exx
	ld a,vD
	exx
	ld (de),a
	inc de
	ei
	ret
GenerateKeySchedule:
	di
	;Input - HL input to key (32 bytes)
	;Output - rkeys (44 bytes) contains the rounded encryption/decryption key
	ld de,key
	ld bc,32
	ldir
	;Key copied
	;Doing 20 rounds
	ld a,P8
	ld (rkeys),a	;S[0]=P(w)
	ld e,1
GenerateKeySchedule_OutsideLoop:
	ld a,e
	dec a
	ld h,99h
	ld l,a
	ld a,(hl)	;A=S[i-1]
	add a,Q8
	inc hl
	ld (hl),a	;S[i]=S[i-1]+Qw
	inc e
	ld a,e
	cp 44
	jr nz,GenerateKeySchedule_OutsideLoop
	;Now ready for A=B=i=j=0
	xor a
	exx
	ld vA,a
	ld vB,a
	exx
	ld (vI),a
	ld (vJ),a
	;Now for v=3 x max {c,2r+4}
	;=132
	;Now for s=1 to v do
	ld a,1
	ld (vS),a
GenerateKeySchedule_SLoop:
	exx
	ld a,(vI)
	ld h,99h
	ld l,a
	ld a,vA
	add a,(hl)
	add a,vB
	;A now contains S[i]+A+B
	;Rotate left 3 bits
	rlca
	rlca
	rlca
	ld vA,a	;A
	exx
	ld e,a
	ld a,(vI)
	ld h,99h
	ld l,a
	ld (hl),e	;S[i]
	;Now we are done with A=S[i]=(S[i]+A+B) <<< 3
	;Next step is B=L[j] = (L[j] +A + B) <<< (A+B)
	ld hl,key
	ld a,(vJ)
	ld b,0
	ld c,a
	add hl,bc
	exx
	ld a,vA
	exx
	ld d,a
	add a,(hl)	;+ L[j]
	ld b,a
	exx
	ld a,vB
	exx
	ld e,a
	add a,b		;A = L[j]+A+B
	exx
	ld vB,a
	exx
	;Get A+B
	ld a,d
	add a,e		;A=A+B
	and 00000111b
	ld b,a
	exx
	ld a,vB
	exx
GenerateKeySchedule_RotateLoop:
	rlca
	djnz GenerateKeySchedule_RotateLoop
	;A contains (L[j]+A+B) <<< (A+B)
	exx
	ld vB,a
	exx
	ld d,a
	ld hl,key
	ld a,(vJ)
	ld b,0
	ld c,a
	add hl,bc
	ld (hl),d	;Set L[j] to value
	;Done with that. Now for i = (i+1) mod (2r + 4)
	;2r + 4 = 44 because r=20
	ld a,(vI)
	inc a
	cp 44
	jr c,GenerateKeySchedule_SkipModulo1
	sub 44
GenerateKeySchedule_SkipModulo1:
	ld (vI),a
	;Done, now below is j = (j+1) mod c
	;We know c = 32
	;
	ld a,(vJ)
	inc a
	and 00011111b
	ld (vJ),a
	;Done with key schedule...} loop up
	ld a,(vS)
	inc a
	ld (vS),a
	cp 133
	jr nz,GenerateKeySchedule_SLoop
	ei
	ret
Multiply:
	;Inputs - A,B two 8-bit numbers to multiply
	;Output - A contains 8-bit answer
	ld c,a
	add a,b
	ld l,a
	adc a,42h
	sub l
	ld h,a
	ld a,c
	ld c,(hl)	;A,B are still same...C is the first part
	sub b
	ld l,a
	sbc a,l
	add a,42h
	ld h,a
	ld a,c
	sub (hl)
	ret
;Multiply:
	;Inputs - A,B two 8-bit numbers to multiply
	;Output - A contains 8-bit answer
;	ld c,a
;	add a,b
;	ld l,a
;	ld a,0
;	adc a,42h
;	ld h,a
;	ld a,c
;	ld c,(hl)	;A,B are still same...C is the first part
;	sub b
;	ld l,a
;	ld a,42h
;	sbc a,0
;	ld h,a
;	ld a,c
;	sub (hl)
;	ret
