VAT_GetFirstNormal:
	;Inputs - none
	;Outputs - HL points to first normal program found in VAT
	;If none found, A will be zero. Otherwise NZ
	ld hl,(progPtr)
VAT_GetNext:
	;Inputs for here are HL pointer to next VAT entry
VAT_GetFirstNormal_Loop:
	ld bc,(pTemp)
	or a
	sbc hl,bc
	jr z,VAT_GetFirstNormal_None
	jr c,VAT_GetFirstNormal_None
	add hl,bc
	ld a,(hl)
	cp 5
	jr z,VAT_GetFirstNormal_CheckProgram
	cp 6
	jr z,VAT_GetFirstNormal_CheckProgram
	ld de,-6
	add hl,de
	ld d,0
	ld e,(hl)
	inc e
	or a
	sbc hl,de
	jr VAT_GetFirstNormal_Loop
VAT_GetFirstNormal_CheckProgram:
	ld de,-5
	add hl,de
	;HL is on page
	dec hl
	dec hl
	ld a,(hl)
	cp 65
	jr c,VAT_GetFirstNormal_SkipHidden
	inc hl
	inc hl
	ld a,(hl)
	or a
	jr z,VAT_GetFirstNormal_CheckProgram_RAM
	;Handle archived program
	;HL is at page byte
	inc hl
	ld d,(hl)
	inc hl
	ld e,(hl)
	push hl
	ex de,hl
	ld bc,64
	ld de,8000h
	B_CALL FlashToRam
	ld hl,8009h
	ld e,(hl)
	ld d,0
	inc e
	add hl,de
	;HL points to size bytes of data
	jr VAT_GetFirstNormal_HLSizeBytes
VAT_GetFirstNormal_CheckProgram_RAM:
	inc hl
	ld d,(hl)
	inc hl
	ld e,(hl)
	push hl
	ex de,hl
	;HL points to size bytes
VAT_GetFirstNormal_HLSizeBytes:
	inc hl
	inc hl
	call VAT_Check_Checksum
	jr nz,VAT_GetFirstNormal_CheckProgram_RAM_Normal
	pop hl
	dec hl
	dec hl
	dec hl
	ld d,0
	ld e,(hl)
	inc e
	or a
	sbc hl,de
	jr VAT_GetFirstNormal_Loop
VAT_GetFirstNormal_CheckProgram_RAM_Normal:
	pop hl
	ld de,3
	add hl,de
	ld a,1
	ret
VAT_GetFirstNormal_SkipHidden:
	inc hl
	ld d,0
	ld e,(hl)
	inc e
	or a
	sbc hl,de
	jr VAT_GetFirstNormal_Loop
VAT_GetFirstNormal_None:
	xor a
	ret
VAT_Check_Checksum:
	;Input - HL one check
	ld de,Checksum_Encrypted
	ld b,10
VAT_Check_Checksum_Loop:
	ld a,(de)
	cp (hl)
	jr nz,VAT_Check_Checksum_Bad
	inc hl
	inc de
	djnz VAT_Check_Checksum_Loop
	xor a
	ret
VAT_Check_Checksum_Bad:
	ld a,1
	or a
	ret
VAT_GetFirstEncrypted:
	;Inputs - none
	;Outputs - HL points to first encrypted program found in VAT
	;If none found, A will be zero. Otherwise NZ
	ld hl,(progPtr)
VAT_GetNext2:
	;Inputs for here are HL pointer to next VAT entry
VAT_GetFirstNormal_Loop2:
	ld bc,(pTemp)
	or a
	sbc hl,bc
	jr z,VAT_GetFirstNormal_None2
	jr c,VAT_GetFirstNormal_None2
	add hl,bc
	ld a,(hl)
	cp 5
	jr z,VAT_GetFirstNormal_CheckProgram2
	cp 6
	jr z,VAT_GetFirstNormal_CheckProgram2
	ld de,-6
	add hl,de
	ld d,0
	ld e,(hl)
	inc e
	or a
	sbc hl,de
	jr VAT_GetFirstNormal_Loop2
VAT_GetFirstNormal_CheckProgram2:
	ld de,-5
	add hl,de
	;HL is on page
	dec hl
	dec hl
	ld a,(hl)
	cp 65
	jr c,VAT_GetFirstNormal_SkipHidden2
	inc hl
	inc hl
	ld a,(hl)
	or a
	jr z,VAT_GetFirstNormal_CheckProgram_RAM2
	;Handle archived program
	;HL is at page byte
	inc hl
	ld d,(hl)
	inc hl
	ld e,(hl)
	push hl
	ex de,hl
	ld bc,64
	ld de,8000h
	B_CALL FlashToRam
	ld hl,8009h
	ld e,(hl)
	ld d,0
	inc e
	add hl,de
	;HL points to size bytes of data
	jr VAT_GetFirstNormal_HLSizeBytes2
VAT_GetFirstNormal_CheckProgram_RAM2:
	inc hl
	ld d,(hl)
	inc hl
	ld e,(hl)
	push hl
	ex de,hl
	;HL points to size bytes
VAT_GetFirstNormal_HLSizeBytes2:
	inc hl
	inc hl
	call VAT_Check_Checksum
	jr z,VAT_GetFirstNormal_CheckProgram_RAM_Normal2
	pop hl
	dec hl
	dec hl
	dec hl
	ld d,0
	ld e,(hl)
	inc e
	or a
	sbc hl,de
	jr VAT_GetFirstNormal_Loop2
VAT_GetFirstNormal_CheckProgram_RAM_Normal2:
	pop hl
	ld de,3
	add hl,de
	ld a,1
	ret
VAT_GetFirstNormal_SkipHidden2:
	inc hl
	ld d,0
	ld e,(hl)
	inc e
	or a
	sbc hl,de
	jr VAT_GetFirstNormal_Loop2
VAT_GetFirstNormal_None2:
	xor a
	ret
